@@ -5,7 +5,7 @@ imdbpy2sql.py script.
 This script puts the data of the plain text data files into a
 SQL database.
 
-Copyright 2005-2007 Davide Alberani <da@erlug.linux.it>
+Copyright 2005-2008 Davide Alberani <da@erlug.linux.it>
                2006 Giuseppe "Cowo" Corbelli <cowo --> lugbs.linux.it>
 
 This program is free software; you can redistribute it and/or modify
@@ -61,6 +61,7 @@ HELP = """imdbpy2sql.py usage:
         # NOTE: --COMPATIBILITY-OPTIONS can be one of:
             --mysql-innodb          insert data into a MySQL MyISAM db,
                                     and then convert it to InnoDB.
+            --mysql-force-myisam    force the creation of MyISAM tables.
             --ms-sqlserver          compatibility mode for Microsoft SQL Server
                                     and SQL Express.
             --sqlite-transactions   uses transactions, to speed-up SQLite.
@@ -73,6 +74,8 @@ HELP = """imdbpy2sql.py usage:
 IMDB_PTDF_DIR = None
 # URI used to connect to the database.
 URI = None
+# Max allowed recursion, inserting data.
+MAX_RECURSION = 10
 # Store custom queries specified on the command line.
 CUSTOM_QUERIES = {}
 # Allowed time specification, for custom queries.
@@ -82,20 +85,27 @@ ALLOWED_TIMES = ('BEGIN', 'BEFORE_DROP', 'BEFORE_CREATE', 'AFTER_CREATE',
                 'AFTER_MOVIES_TODB', 'BEFORE_PERSONS_TODB',
                 'AFTER_PERSONS_TODB','BEFORE_SQLDATA_TODB',
                 'AFTER_SQLDATA_TODB', 'BEFORE_AKAMOVIES_TODB',
-                'AFTER_AKAMOVIES_TODB', 'BEFORE_EVERY_TODB',
+                'AFTER_AKAMOVIES_TODB', 'BEFORE_CHARACTERS_TODB',
+                'AFTER_CHARACTERS_TODB', 'BEFORE_EVERY_TODB',
                 'AFTER_EVERY_TODB')
 
 # Shortcuts for some compatibility options.
+MYSQLFORCEMYISAM_OPTS = ['-e',
+        'AFTER_CREATE:FOR_EVERY_TABLE:ALTER TABLE %(table)s ENGINE=MyISAM;']
 MYSQLINNODB_OPTS = ['-e',
         'AFTER_CREATE:FOR_EVERY_TABLE:ALTER TABLE %(table)s ENGINE=MyISAM;',
         '-e', 'END:FOR_EVERY_TABLE:ALTER TABLE %(table)s ENGINE=InnoDB;']
 SQLSERVER_OPTS = ['-e', 'BEFORE_EVERY_TODB:SET IDENTITY_INSERT %(table)s ON;',
         '-e', 'AFTER_EVERY_TODB:SET IDENTITY_INSERT %(table)s OFF;']
 SQLITE_OPTS = ['-e', 'BEFORE_EVERY_TODB:BEGIN TRANSACTION;',
-        '-e', 'AFTER_EVERY_TODB:COMMIT;']
+        '-e', 'AFTER_EVERY_TODB:COMMIT;',
+        '-e', 'BEFORE_INDEXES:BEGIN TRANSACTION;',
+        'e', 'END:COMMIT;']
 
 if '--mysql-innodb' in sys.argv[1:]:
     sys.argv += MYSQLINNODB_OPTS
+if '--mysql-force-myisam' in sys.argv[1:]:
+    sys.argv += MYSQLFORCEMYISAM_OPTS
 if '--ms-sqlserver' in sys.argv[1:]:
     sys.argv += SQLSERVER_OPTS
 if '--sqlite-transactions' in sys.argv[1:]:
@@ -107,6 +117,7 @@ try:
                                                 ['uri=', 'data=', 'execute=',
                                                 'mysql-innodb', 'ms-sqlserver',
                                                 'sqlite-transactions',
+                                                'mysql-force-myisam',
                                                 'help'])
 except getopt.error, e:
     print 'Troubles with arguments.'
@@ -128,11 +139,13 @@ for opt in optlist:
             continue
         if when == 'BEFORE_EVERY_TODB':
             for nw in ('BEFORE_MOVIES_TODB', 'BEFORE_PERSONS_TODB',
-                        'BEFORE_SQLDATA_TODB', 'BEFORE_AKAMOVIES_TODB'):
+                        'BEFORE_SQLDATA_TODB', 'BEFORE_AKAMOVIES_TODB',
+                        'BEFORE_CHARACTERS_TODB'):
                 CUSTOM_QUERIES.setdefault(nw, []).append(cmd)
         elif when == 'AFTER_EVERY_TODB':
             for nw in ('AFTER_MOVIES_TODB', 'AFTER_PERSONS_TODB',
-                        'AFTER_SQLDATA_TODB', 'AFTER_AKAMOVIES_TODB'):
+                        'AFTER_SQLDATA_TODB', 'AFTER_AKAMOVIES_TODB',
+                        'AFTER_CHARACTERS_TODB'):
                 CUSTOM_QUERIES.setdefault(nw, []).append(cmd)
         else:
             CUSTOM_QUERIES.setdefault(when, []).append(cmd)
@@ -150,6 +163,51 @@ if URI is None:
     print HELP
     sys.exit(2)
 
+# Some warnings and notices.
+URIlower = URI.lower()
+
+if URIlower.startswith('mysql'):
+    if '--mysql-force-myisam' in sys.argv[1:] and \
+            '--mysql-innodb' in sys.argv[1:]:
+        print '\nWARNING: there is no sense in mixing the --mysql-innodb and\n'\
+                '--mysql-force-myisam command line options!\n'
+    elif '--mysql-innodb' in sys.argv[1:]:
+        print "\nNOTICE: you've specified the --mysql-innodb command line\n"\
+                "option; you should do this ONLY IF your system uses InnoDB\n"\
+                "tables or you really want to use InnoDB; if you're running\n"\
+                "a MyISAM-based database, please omit any option; if you\n"\
+                "want to force MyISAM usage on a InnoDB-based database,\n"\
+                "try the --mysql-force-myisam command line option, instead.\n"
+    elif '--mysql-force-myisam' in sys.argv[1:]:
+        print "\nNOTICE: you've specified the --mysql-force-myisam command\n"\
+                "line option; you should do this ONLY IF your system uses\n"\
+                "InnoDB tables and you want to use MyISAM tables, instead.\n"
+    else:
+        print "\nNOTICE: IF you're using InnoDB tables, data insertion can\n"\
+                "be very slow; you can switch to MyISAM tables - forcing it\n"\
+                "with the --mysql-force-myisam option - OR use the\n"\
+                "--mysql-innodb command line option, but DON'T USE these if\n"\
+                "you're already working on MyISAM tables, because it will\n"\
+                "force MySQL to use InnoDB, and performances will be poor.\n"
+elif URIlower.startswith('mssql') and \
+        '--ms-sqlserver' not in sys.argv[1:]:
+    print "\nWARNING: you're using MS SQLServer without the --ms-sqlserver\n"\
+            "command line option: if something goes wrong, try using it.\n"
+elif URIlower.startswith('sqlite') and \
+        '--sqlite-transactions' not in sys.argv[1:]:
+    print "\nWARNING: you're using SQLite without the --sqlite-transactions\n"\
+            "command line option: you'll have very poor performances!  Try\n"\
+            "using it.\n"
+if ('--mysql-force-myisam' in sys.argv[1:] and
+        not URIlower.startswith('mysql')) or ('--mysql-innodb' in
+        sys.argv[1:] and not URIlower.startswith('mysql')) or ('--ms-sqlserver'
+        in sys.argv[1:] and not URIlower.startswith('mssql')) or \
+        ('--sqlite-transactions' in sys.argv[1:] and
+        not URIlower.startswith('sqlite')):
+    print "\nWARNING: you've specified command line options that don't\n"\
+            "belong to the database server you're using: proceed at your\n"\
+            "own risk!\n"
+
 # Connect to the database.
 conn = setConnection(URI)
 # Extract exceptions to trap.
@@ -488,7 +546,7 @@ class _BaseCache(dict):
         """Flush to the database."""
         if self._flushing: return
         self._flushing = 1
-        if _recursionLevel >= 5:
+        if _recursionLevel >= MAX_RECURSION:
             print 'WARNING recursion level exceded trying to flush data'
             print 'WARNING this batch of data is lost (%s).' % self.className
             self._tmpDict.clear()
@@ -674,6 +732,7 @@ class MoviesCache(_BaseCache):
 
 class PersonsCache(_BaseCache):
     """Manage the persons list."""
+    className = 'PersonsCache'
     counter = counter()
 
     def __init__(self, *args, **kwds):
@@ -794,6 +853,7 @@ class SQLData(dict):
         self.sqlString = sqlString
         self.converter = converter
         self._recursionLevel = 1
+        self._table = table
         self._table_name = tableName(table)
         for k, v in d.items(): self[k] = v
 
@@ -817,7 +877,7 @@ class SQLData(dict):
         CACHE_MID.flush(quiet=1)
         CACHE_PID.flush(quiet=1)
         if _resetRecursion: self._recursionLevel = 1
-        if self._recursionLevel >= 5:
+        if self._recursionLevel >= MAX_RECURSION:
             print 'WARNING recursion level exceded trying to flush data'
             print 'WARNING this batch of data is lost.'
             self.clear()
@@ -841,7 +901,8 @@ class SQLData(dict):
             print ' * TOO MANY DATA (%s items), SPLITTING (run #%d)...' % \
                     (len(self), self._recursionLevel)
             self._recursionLevel += 1
-            newdata = self.__class__(sqlString=self.sqlString,
+            newdata = self.__class__(table=self._table,
+                                    sqlString=self.sqlString,
                                     converter=self.converter)
             newdata._recursionLevel = self._recursionLevel
             newflushEvery = self.flushEvery / 2
@@ -997,7 +1058,7 @@ def doCast(fp, roleid, rolename):
                             pass
         movieid = CACHE_MID.addUnique(title)
         if role is not None:
-            roles = role.split('/')
+            roles = filter(None, [x.strip() for x in role.split('/')])
             for role in roles:
                 cid = CACHE_CID.addUnique(role)
                 sqldata.add((pid, movieid, cid, note, order))
@@ -1011,8 +1072,13 @@ def doCast(fp, roleid, rolename):
     print 'CLOSING %s...' % rolename
 
 
-def castLists():
+def castLists(_charIDsList=None):
     """Read files listed in the 'role' column of the 'roletypes' table."""
+    # _charIDsList is a dirty hack to allow us to get rid of
+    # CACHE_CID as soon as possible (right after that actresses.list.gz
+    # is processed).
+    if _charIDsList is None:
+        _charIDsList = []
     for rt in RoleType.select():
         roleid = rt.id
         rolename = fname = rt.role
@@ -1027,12 +1093,22 @@ def castLists():
             f = SourceFile(fname, start=CAST_START, stop=CAST_STOP)
         except IOError:
             if rolename == 'actress':
+                try:
+                    restoreImdbID(_charIDsList, CharName)
+                    del _charIDsList
+                except Exception, e:
+                    print 'WARNING: failed to restore imdbIDs for characters:',e
                 CACHE_CID.flush()
                 CACHE_CID.clear()
             continue
         doCast(f, roleid, rolename)
         f.close()
         if rolename == 'actress':
+            try:
+                restoreImdbID(_charIDsList, CharName)
+                del _charIDsList
+            except Exception, e:
+                print 'WARNING: failed to restore imdbIDs for characters:', e
             CACHE_CID.flush()
             CACHE_CID.clear()
         t('castLists(%s)' % rolename)
@@ -1593,6 +1669,7 @@ def completeCast():
 CACHE_MID = MoviesCache()
 CACHE_PID = PersonsCache()
 CACHE_CID = CharactersCache()
+CACHE_CID.className = 'CharactersCache'
 
 
 def _cmpfunc(x, y):
@@ -1632,7 +1709,8 @@ def notNULLimdbID(cls):
     """Return a list of dictionaries for titles or names for which a
     imdbID is present in the database."""
     if cls is Title: cname = 'movies'
-    else: cname = 'people'
+    elif cls is Name: cname = 'people'
+    else: cname = 'characters'
     print 'SAVING imdbID values for %s...' % cname,
     sys.stdout.flush()
     try:
@@ -1666,9 +1744,12 @@ def restoreImdbID(tons, cls):
     if cls is Title:
         CACHE = CACHE_MID
         cname = 'movies'
-    else:
+    elif cls is Name:
         CACHE = CACHE_PID
         cname = 'people'
+    else:
+        CACHE = CACHE_CID
+        cname = 'characters'
     print 'RESTORING imdbID values for %s...' % cname,
     sys.stdout.flush()
     count = 0
@@ -1737,14 +1818,19 @@ def run():
     # Storing imdbIDs for movies and persons.
     try:
         movies_imdbIDs = notNULLimdbID(Title)
-    except:
+    except Exception, e:
         movies_imdbIDs = []
-        print 'WARNING: failed to read imdbIDs for movies'
+        print 'WARNING: failed to read imdbIDs for movies: %s' % e
     try:
         people_imdbIDs = notNULLimdbID(Name)
-    except:
+    except Exception, e:
         people_imdbIDs = []
-        print 'WARNING: failed to read imdbIDs for people'
+        print 'WARNING: failed to read imdbIDs for people: %s' % e
+    try:
+        characters_imdbIDs = notNULLimdbID(CharName)
+    except Exception, e:
+        characters_imdbIDs = []
+        print 'WARNING: failed to read imdbIDs for characters: %s' % e
 
     executeCustomQueries('BEFORE_DROP')
 
@@ -1775,13 +1861,15 @@ def run():
     # Comment readMovieList() and uncomment the following two lines
     # to keep the current info in the name and title tables.
     ##CACHE_MID.populate()
-    ##CACHE_PID.populate()
     t('readMovieList()')
 
     # actors, actresses, producers, writers, cinematographers, composers,
     # costume-designers, directors, editors, miscellaneous,
     # production-designers.
-    castLists()
+    castLists(_charIDsList=characters_imdbIDs)
+    ##CACHE_PID.populate()
+    ##CACHE_CID.populate()
+    del characters_imdbIDs
 
     # Aka names and titles.
     doAkaNames()
@@ -1824,13 +1912,13 @@ def run():
     try:
         restoreImdbID(movies_imdbIDs, Title)
         del movies_imdbIDs
-    except:
-        print 'WARNING: failed to restore imdbIDs for movies'
+    except Exception, e:
+        print 'WARNING: failed to restore imdbIDs for movies: %s' % e
     try:
         restoreImdbID(people_imdbIDs, Name)
         del people_imdbIDs
-    except:
-        print 'WARNING: failed to restore imdbIDs for people'
+    except Exception, e:
+        print 'WARNING: failed to restore imdbIDs for people: %s' % e
 
     # Flush caches.
     CACHE_MID.flush()
