@@ -6,7 +6,7 @@ a person from the IMDb database.
 It can fetch data through different media (e.g.: the IMDb web pages,
 a local installation, a SQL database, etc.)
 
-Copyright 2004-2007 Davide Alberani <da@erlug.linux.it>
+Copyright 2004-2008 Davide Alberani <da@erlug.linux.it>
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -23,8 +23,12 @@ along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 """
 
-__all__ = ['IMDb', 'IMDbError', 'Movie', 'Person', 'Character']
-__version__ = VERSION = '3.4'
+__all__ = ['IMDb', 'IMDbError', 'Movie', 'Person', 'Character',
+            'available_access_systems']
+__version__ = VERSION = '3.5'
+
+# Import compatibility module.
+import _compat
 
 import sys, os, ConfigParser
 from types import UnicodeType, TupleType, ListType, MethodType
@@ -95,6 +99,8 @@ class ConfigParserWithCase(ConfigParser.ConfigParser):
 
     def _manageValue(self, value):
         """Custom substitutions for values."""
+        if not isinstance(value, (str, unicode)):
+            return value
         vlower = value.lower()
         if vlower in self._boolean_states:
             return self._boolean_states[vlower]
@@ -111,10 +117,10 @@ class ConfigParserWithCase(ConfigParser.ConfigParser):
     def items(self, section, *args, **kwds):
         """Return a list of (key, value) tuples of items of the
         given section."""
-        if not self.has_section(section):
+        if section != 'DEFAULT' and not self.has_section(section):
             return []
-        items = ConfigParser.ConfigParser.items(self, section, *args, **kwds)
-        return [(key, self._manageValue(value)) for key, value in items]
+        keys = ConfigParser.ConfigParser.options(self, section)
+        return [(k, self.get(section, k, *args, **kwds)) for k in keys]
 
     def getDict(self, section):
         """Return a dictionary of items of the specified section."""
@@ -172,6 +178,33 @@ def IMDb(accessSystem=None, *arguments, **keywords):
                             % accessSystem
 
 
+def available_access_systems():
+    """Return the list of available data access systems."""
+    asList = []
+    # XXX: trying to import modules is a good thing?
+    try:
+        from parser.http import IMDbHTTPAccessSystem
+        asList += ['http', 'httpThin']
+    except ImportError:
+        pass
+    try:
+        from parser.mobile import IMDbMobileAccessSystem
+        asList.append('mobile')
+    except ImportError:
+        pass
+    try:
+        from parser.local import IMDbLocalAccessSystem
+        asList.append('local')
+    except ImportError:
+        pass
+    try:
+        from parser.sql import IMDbSqlAccessSystem
+        asList.append('sql')
+    except ImportError:
+        pass
+    return asList
+
+
 # XXX: I'm not sure this is a good guess.
 #      I suppose that an argument of the IMDb function can be used to
 #      set a default encoding for the output, and then Movie, Person and
@@ -191,7 +224,8 @@ class IMDbBase:
     # in the subclasses).
     accessSystem = 'UNKNOWN'
 
-    def __init__(self, defaultModFunct=None, *arguments, **keywords):
+    def __init__(self, defaultModFunct=None, results=20,
+                *arguments, **keywords):
         """Initialize the access system.
         If specified, defaultModFunct is the function used by
         default by the Person, Movie and Character objects, when
@@ -200,6 +234,14 @@ class IMDbBase:
         # The function used to output the strings that need modification (the
         # ones containing references to movie titles and person names).
         self._defModFunct = defaultModFunct
+        # Number of results to get.
+        try:
+            results = int(results)
+        except (TypeError, ValueError):
+            results = 20
+        if results < 1:
+            results = 20
+        self._results = results
 
     def _normalize_movieID(self, movieID):
         """Normalize the given movieID."""
@@ -281,9 +323,11 @@ class IMDbBase:
         #      subclass, somewhere under the imdb.parser package.
         raise NotImplementedError, 'override this method'
 
-    def search_movie(self, title, results=20):
+    def search_movie(self, title, results=None):
         """Return a list of Movie objects for a query for the given title.
         The results argument is the maximum number of results to return."""
+        if results is None:
+            results = self._results
         try:
             results = int(results)
         except (ValueError, OverflowError):
@@ -325,10 +369,12 @@ class IMDbBase:
         #      subclass, somewhere under the imdb.parser package.
         raise NotImplementedError, 'override this method'
 
-    def search_person(self, name, results=20):
+    def search_person(self, name, results=None):
         """Return a list of Person objects for a query for the given name.
 
         The results argument is the maximum number of results to return."""
+        if results is None:
+            results = self._results
         try:
             results = int(results)
         except (ValueError, OverflowError):
@@ -368,10 +414,12 @@ class IMDbBase:
         #      subclass, somewhere under the imdb.parser package.
         raise NotImplementedError, 'override this method'
 
-    def search_character(self, name, results=20):
+    def search_character(self, name, results=None):
         """Return a list of Character objects for a query for the given name.
 
         The results argument is the maximum number of results to return."""
+        if results is None:
+            results = self._results
         try:
             results = int(results)
         except (ValueError, OverflowError):
