@@ -3,7 +3,7 @@ utils module (imdb package).
 
 This module provides basic utilities for the imdb package.
 
-Copyright 2004-2007 Davide Alberani <da@erlug.linux.it>
+Copyright 2004-2008 Davide Alberani <da@erlug.linux.it>
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -26,7 +26,7 @@ from types import UnicodeType, StringType, ListType, TupleType, DictType
 from copy import copy, deepcopy
 from time import strptime, strftime
 
-from imdb._exceptions import IMDbParserError
+from imdb._exceptions import IMDbParserError, IMDbError
 
 # The regular expression for the "long" year format of IMDb, like
 # "(1998)" and "(1986/II)", where the optional roman number (that I call
@@ -40,7 +40,6 @@ re_index = re.compile(r'^\(([IVXLCDM]+)\)$')
 
 # Match the number of episodes.
 re_episodes = re.compile('\s?\((\d+) episodes\)', re.I)
-
 re_episode_info = re.compile(r'{(.+?)?\s?(\([0-9\?]{4}-[0-9\?]{1,2}-[0-9\?]{1,2}\))?\s?(\(#[0-9]+\.[0-9]+\))?}')
 
 # Common suffixes in surnames.
@@ -50,14 +49,15 @@ _sname_suffixes = ('de', 'la', 'der', 'den', 'del', 'y', 'da', 'van',
 def canonicalName(name):
     """Return the given name in canonical "Surname, Name" format.
     It assumes that name is in the 'Name Surname' format."""
-    # XXX: some statistics (over 1852406 names):
-    #      - just a surname:                 51921
-    #      - single surname, single name:  1792759
-    #      - composed surname, composed name: 7726
-    #      - composed surname, single name:  55623
-    #        (2: 49259, 3: 5502, 4: 551)
-    #      - single surname, composed name: 186604
-    #        (2: 178315, 3: 6573, 4: 1219, 5: 352)
+    # XXX: some statistics (as of 17 Apr 2008, over 2288622 names):
+    #      - just a surname:                 69476
+    #      - single surname, single name:  2209656
+    #      - composed surname, composed name: 9490
+    #      - composed surname, single name:  67606
+    #        (2: 59764, 3: 6862, 4: 728)
+    #      - single surname, composed name: 242310
+    #        (2: 229467, 3: 9901, 4: 2041, 5: 630)
+    #      - Jr.: 8025
     # Don't convert names already in the canonical format.
     if name.find(', ') != -1: return name
     sname = name.split(' ')
@@ -154,25 +154,25 @@ def build_name(name_dict, canonical=0):
 # even in some American slangs.  Unfortunately it's also a preposition in
 # Italian, and it's widely used in Mandarin (for whatever it means!).
 # Running a script over the whole list of titles (and aliases), I've found
-# that 'da' is used as an article only 20 times, and as another thing 255
+# that 'da' is used as an article only 23 times, and as another thing 298
 # times, so I've decided to _always_ consider 'da' as a non article.
 #
 # Here is a list of words that are _never_ considered as articles, complete
 # with the cound of times they are used in a way or another:
-# 'en' (314 vs 507), 'to' (236 vs 589), 'as' (183 vs 231), 'et' (67 vs 79),
-# 'des' (69 vs 123), 'al' (57 vs 247), 'egy' (28 vs 32), 'ye' (14 vs 55),
-# 'da' (20 vs 255), "'n" (7 vs 12)
+# 'en' (376 vs 594), 'to' (399 vs 727), 'as' (198 vs 276), 'et' (79 vs 99),
+# 'des' (75 vs 150), 'al' (78 vs 304), 'ye' (14 vs 70),
+# 'da' (23 vs 298), "'n" (8 vs 12)
 #
-# I've left in the list 'i' (1614 vs 1707) and 'uno' (49 vs 51)
+# I've left in the list 'i' (1939 vs 2151) and 'uno' (52 vs 56)
 # I'm not sure what '-al' is, and so I've left it out...
 #
 # List of articles:
 _articles = ('the', 'la', 'a', 'die', 'der', 'le', 'el',
-            "l'", 'il', 'das', 'les', 'o', 'ein', 'i', 'un', 'los', 'de',
-            'an', 'una', 'las', 'eine', 'den', 'gli', 'het', 'os', 'lo',
-            'az', 'det', 'ha-', 'een', 'ang', 'oi', 'ta', 'al-', 'dem',
-            'mga', 'uno', "un'", 'ett', u'\xcf', 'eines', u'\xc7', 'els',
-            u'\xd4\xef', u'\xcf\xe9')
+            "l'", 'il', 'das', 'les', 'i', 'o', 'ein', 'un', 'de', 'los',
+            'an', 'una', 'las', 'eine', 'den', 'het', 'gli', 'lo', 'os',
+            'ang', 'oi', 'az', 'een', 'ha-', 'det', 'ta', 'al-',
+            'mga', "un'", 'uno', 'ett', 'dem', 'egy', 'els', 'eines', u'\xcf',
+            u'\xc7', u'\xd4\xef', u'\xcf\xe9')
 
 # Articles in a dictionary.
 _articlesDict = dict([(x, x) for x in _articles])
@@ -326,14 +326,14 @@ def analyze_title(title, canonical=None,
                 episode_d['episode'] = epn
         return episode_d
     # First of all, search for the kind of show.
-    # XXX: Number of entries at 18 Mar 2006:
-    #      movie:        344,892
-    #      episode:      272,862
-    #      tv movie:      53,269
-    #      tv series:     37,065
-    #      video movie:   44,062
-    #      tv mini series: 4,757
-    #      video game:     4,472
+    # XXX: Number of entries at 17 Apr 2008:
+    #      movie:        379,871
+    #      episode:      483,832
+    #      tv movie:      61,119
+    #      tv series:     44,795
+    #      video movie:   57,915
+    #      tv mini series: 5,497
+    #      video game:     5,490
     #      More up-to-date statistics: http://us.imdb.com/database_statistics
     if title.endswith('(TV)'):
         kind = u'tv movie'
@@ -550,10 +550,8 @@ def cmpMovies(m1, m2):
 
 def cmpPeople(p1, p2):
     """Compare two people by billingPos, name and imdbIndex."""
-    p1b = p1.billingPos
-    if p1b is None: p1b = _last
-    p2b = p2.billingPos
-    if p2b is None: p2b = _last
+    p1b = getattr(p1, 'billingPos', None) or _last
+    p2b = getattr(p2, 'billingPos', None) or _last
     if p1b > p2b: return 1
     if p1b < p2b: return -1
     p1n = p1.get('canonical name', _last)
@@ -570,37 +568,44 @@ def cmpPeople(p1, p2):
     return 0
 
 
-# References to titles and names.
+# References to titles, names and characters.
 # XXX: find better regexp!
 re_titleRef = re.compile(r'_(.+?(?: \([0-9\?]{4}(?:/[IVXLCDM]+)?\))?(?: \(mini\)| \(TV\)| \(V\)| \(VG\))?)_ \(qv\)')
 # FIXME: doesn't match persons with ' in the name.
 re_nameRef = re.compile(r"'([^']+?)' \(qv\)")
+# XXX: good choice?  Are there characters with # in the name?
+re_characterRef = re.compile(r"#([^']+?)# \(qv\)")
 
 # Functions used to filter the text strings.
-def modNull(s, titlesRefs, namesRefs):
+def modNull(s, titlesRefs, namesRefs, charactersRefs):
     """Do nothing."""
     return s
 
-def modClearTitleRefs(s, titlesRefs, namesRefs):
+def modClearTitleRefs(s, titlesRefs, namesRefs, charactersRefs):
     """Remove titles references."""
     return re_titleRef.sub(r'\1', s)
 
-def modClearNameRefs(s, titlesRefs, namesRefs):
+def modClearNameRefs(s, titlesRefs, namesRefs, charactersRefs):
     """Remove names references."""
     return re_nameRef.sub(r'\1', s)
 
-def modClearRefs(s, titlesRefs, namesRefs):
-    """Remove both titles and names references."""
-    s = modClearTitleRefs(s, {}, {})
-    return modClearNameRefs(s, {}, {})
+def modClearCharacterRefs(s, titlesRefs, namesRefs, charactersRefs):
+    """Remove characters references"""
+    return re_characterRef.sub(r'\1', s)
+
+def modClearRefs(s, titlesRefs, namesRefs, charactersRefs):
+    """Remove titles, names and characters references."""
+    s = modClearTitleRefs(s, {}, {}, {})
+    s = modClearCharacterRefs(s, {}, {}, {})
+    return modClearNameRefs(s, {}, {}, {})
 
 
-def modifyStrings(o, modFunct, titlesRefs, namesRefs):
+def modifyStrings(o, modFunct, titlesRefs, namesRefs, charactersRefs):
     """Modify a string (or string values in a dictionary or strings
     in a list), using the provided modFunct function and titlesRefs
-    and namesRefs references dictionaries."""
+    namesRefs and charactersRefs references dictionaries."""
     if isinstance(o, (UnicodeType, StringType)):
-        return modFunct(o, titlesRefs, namesRefs)
+        return modFunct(o, titlesRefs, namesRefs, charactersRefs)
     elif isinstance(o, (ListType, TupleType)):
         _stillorig = 1
         if isinstance(o, ListType): keys = xrange(len(o))
@@ -611,9 +616,10 @@ def modifyStrings(o, modFunct, titlesRefs, namesRefs):
                 if _stillorig:
                     o = copy(o)
                     _stillorig = 0
-                o[i] = modFunct(v, titlesRefs, namesRefs)
+                o[i] = modFunct(v, titlesRefs, namesRefs, charactersRefs)
             elif isinstance(v, (ListType, TupleType)):
-                modifyStrings(o[i], modFunct, titlesRefs, namesRefs)
+                modifyStrings(o[i], modFunct, titlesRefs, namesRefs,
+                            charactersRefs)
     return o
 
 
@@ -640,8 +646,20 @@ def date_and_notes(s):
     return s, notes
 
 
-class _Container:
-    """Base class for Movie and Person classes."""
+class RolesList(list):
+    """A list of Person or Character instances, used for the currentRole
+    property."""
+    def __unicode__(self):
+        return u' / '.join([unicode(x) for x in self])
+
+    def __str__(self):
+        # FIXME: does it make sense at all?  Return a unicode doesn't
+        #        seem right, in __str__.
+        return u' / '.join([unicode(x).encode('utf8') for x in self])
+
+
+class _Container(object):
+    """Base class for Movie, Person and Character classes."""
      # The default sets of information retrieved.
     default_info = ()
 
@@ -653,51 +671,135 @@ class _Container:
 
     cmpFunct = None
 
-    def __init__(self, myID=None, data=None, currentRole=u'', notes=u'',
+    def __init__(self, myID=None, data=None, notes=u'',
+                currentRole=u'', roleID=None, roleIsPerson=False,
                 accessSystem=None, titlesRefs=None, namesRefs=None,
-                modFunct=None, *args, **kwds):
-        """Initialize a Movie or a Person object.
+                charactersRefs=None, modFunct=None, *args, **kwds):
+        """Initialize a Movie, Person or Character object.
         *myID* -- your personal identifier for this object.
         *data* -- a dictionary used to initialize the object.
-        *currentRole* -- a string representing the current role or duty
-                        of a person in this/a movie.
         *notes* -- notes for the person referred in the currentRole
                     attribute; e.g.: '(voice)' or the alias used in the
                     movie credits.
         *accessSystem* -- a string representing the data access system used.
+        *currentRole* -- a Character instance representing the current role
+                         or duty of a person in this movie, or a Person
+                         object representing the actor/actress who played
+                         a given character in a Movie.  If a string is
+                         passed, an object is automatically build.
+        *roleID* -- if available, the characterID/personID of the currentRole
+                    object.
+        *roleIsPerson* -- when False (default) the currentRole is assumed
+                          to be a Character object, otherwise a Person.
         *titlesRefs* -- a dictionary with references to movies.
         *namesRefs* -- a dictionary with references to persons.
+        *charactersRefs* -- a dictionary with references to characters.
         *modFunct* -- function called returning text fields.
         """
         self.reset()
+        self.accessSystem = accessSystem
         self.myID = myID
         if data is None: data = {}
         self.set_data(data, override=1)
-        self.currentRole = currentRole
         self.notes = notes
-        self.accessSystem = accessSystem
         if titlesRefs is None: titlesRefs = {}
         self.update_titlesRefs(titlesRefs)
         if namesRefs is None: namesRefs = {}
         self.update_namesRefs(namesRefs)
+        if charactersRefs is None: charactersRefs = {}
+        self.update_charactersRefs(charactersRefs)
         self.set_mod_funct(modFunct)
         self.keys_tomodify = {}
         for item in self.keys_tomodify_list:
             self.keys_tomodify[item] = None
+        self._roleIsPerson = roleIsPerson
+        if not roleIsPerson:
+            from imdb.Character import Character
+            self._roleClass = Character
+        else:
+            from imdb.Person import Person
+            self._roleClass = Person
+        self.currentRole = currentRole
+        if roleID:
+            self.roleID = roleID
         self._init(*args, **kwds)
 
+    def _get_roleID(self):
+        """Return the characterID or personID of the currentRole object."""
+        if not self.__role:
+            return None
+        if isinstance(self.__role, list):
+            return [x.getID() for x in self.__role]
+        return self.currentRole.getID()
+
+    def _set_roleID(self, roleID):
+        """Set the characterID or personID of the currentRole object."""
+        if not self.__role:
+            # XXX: needed?  Just ignore it?  It's probably safer to
+            #      ignore it, to prevent some bugs in the parsers.
+            raise IMDbError, "Can't set ID of an empty Character/Person object."
+        if not self._roleIsPerson:
+            if not isinstance(roleID, (list, tuple)):
+                self.currentRole.characterID = roleID
+            else:
+                for index, item in enumerate(roleID):
+                    self.__role[index].characterID = item
+        else:
+            if not isinstance(roleID, (list, tuple)):
+                self.currentRole.personID = roleID
+            else:
+                for index, item in enumerate(roleID):
+                    self.__role[index].personID = item
+
+    roleID = property(_get_roleID, _set_roleID,
+                doc="the characterID or personID of the currentRole object.")
+
+    def _get_currentRole(self):
+        """Return a Character or Person instance."""
+        if self.__role:
+            return self.__role
+        return self._roleClass(name=u'', accessSystem=self.accessSystem,
+                                modFunct=self.modFunct)
+
+    def _set_currentRole(self, role):
+        """Set self.currentRole to a Character or Person instance."""
+        if isinstance(role, (UnicodeType, StringType)):
+            if not role:
+                self.__role = None
+            else:
+                self.__role = self._roleClass(name=role, modFunct=self.modFunct,
+                                        accessSystem=self.accessSystem)
+        elif isinstance(role, (list, tuple)):
+            self.__role = RolesList()
+            for item in role:
+                if isinstance(item, (UnicodeType, StringType)):
+                    self.__role.append(self._roleClass(name=item,
+                                        accessSystem=self.accessSystem,
+                                        modFunct=self.modFunct))
+                else:
+                    self.__role.append(item)
+            if not self.__role:
+                self.__role = None
+        else:
+            self.__role = role
+
+    currentRole = property(_get_currentRole, _set_currentRole,
+                            doc="The role of a Person in a Movie" + \
+                            " or the interpreter of a Character in a Movie.")
+
     def _init(self, **kwds): pass
 
     def reset(self):
         """Reset the object."""
         self.data = {}
         self.myID = None
-        self.currentRole = u''
         self.notes = u''
         self.titlesRefs = {}
         self.namesRefs = {}
+        self.charactersRefs = {}
         self.modFunct = modClearRefs
         self.current_info = []
+        self.__role = None
         self._reset()
 
     def _reset(self): pass
@@ -705,11 +807,12 @@ class _Container:
     def clear(self):
         """Reset the dictionary."""
         self.data.clear()
-        self.currentRole = u''
         self.notes = u''
         self.titlesRefs = {}
         self.namesRefs = {}
+        self.charactersRefs = {}
         self.current_info = []
+        self.__role = None
         self._clear()
 
     def _clear(self): pass
@@ -752,6 +855,14 @@ class _Container:
         """Return the dictionary with the references to names."""
         return self.namesRefs
 
+    def update_charactersRefs(self, charactersRefs):
+        """Update the dictionary with the references to characters."""
+        self.charactersRefs.update(charactersRefs)
+
+    def get_charactersRefs(self):
+        """Return the dictionary with the references to characters."""
+        return self.charactersRefs
+
     def set_data(self, data, override=0):
         """Set the movie data to the given dictionary; if 'override' is
         set, the previous data is removed, otherwise the two dictionary
@@ -763,11 +874,11 @@ class _Container:
             self.data = data
 
     def getID(self):
-        """Return movie or person ID."""
+        """Return movieID, personID or characterID."""
         raise NotImplementedError, 'override this method'
 
     def __cmp__(self, other):
-        """Compare two Movie or Person objects."""
+        """Compare two Movie, Person or Character objects."""
         # XXX: raise an exception?
         if self.cmpFunct is None: return -1
         if not isinstance(other, self.__class__): return -1
@@ -780,18 +891,21 @@ class _Container:
         if theID is not None and self.accessSystem not in ('UNKNOWN', None):
             # Handle 'http' and 'mobile' as they are the same access system.
             acs = self.accessSystem
-            if acs == 'mobile': acs = 'http'
+            if acs in ('mobile', 'httpThin'):
+                acs = 'http'
             s4h = '%s:%s' % (acs, theID)
         else:
             s4h = repr(self)
         return hash(s4h)
 
     def isSame(self, other):
+        """Return True if the two represent the same object."""
         if not isinstance(other, self.__class__): return 0
         if hash(self) == hash(other): return 1
         return 0
 
     def __len__(self):
+        """Number of items in the data dictionary."""
         return len(self.data)
 
     def _getitem(self, key):
@@ -809,8 +923,16 @@ class _Container:
         rawData = self.data[key]
         if self.keys_tomodify.has_key(key) and \
                 self.modFunct not in (None, modNull):
-            return modifyStrings(rawData, self.modFunct, self.titlesRefs,
-                                self.namesRefs)
+            try:
+                return modifyStrings(rawData, self.modFunct, self.titlesRefs,
+                                    self.namesRefs, self.charactersRefs)
+            except RuntimeError, e:
+                # Symbian/python 2.2 has a poor regexp implementation.
+                import warnings
+                warnings.warn('RuntimeError in '
+                        "imdb.utils._Container.__getitem__; if it's not "
+                        "a recursion limit exceeded or we're not running "
+                        "in a Symbian environment, it's a bug:\n%s" % e)
         return rawData
 
     def __setitem__(self, key, item):
@@ -909,8 +1031,8 @@ class _Container:
 
 def flatten(seq, toDescend=(ListType, DictType, TupleType),
             yieldDictKeys=0, onlyKeysType=(_Container,), scalar=None):
-    """Iterate over nested lists and dictionaries; toDescend is a type
-    of a tuple of types to be considered non-scalar; if yieldDictKeys is
+    """Iterate over nested lists and dictionaries; toDescend is a list
+    or a tuple of types to be considered non-scalar; if yieldDictKeys is
     true, also dictionaries' keys are yielded; if scalar is not None, only
     items of the given type(s) are yielded."""
     if not isinstance(seq, toDescend):
@@ -938,3 +1060,4 @@ def flatten(seq, toDescend=(ListType, DictType, TupleType),
                                 onlyKeysType=onlyKeysType, scalar=scalar):
                     yield i
 
+
